from PyQt4.QtGui import *
from PyQt4.QtCore import pyqtSignal, QModelIndex, pyqtSlot, QRegExp, QTimer

import re
import traceback


def parse_card_info(line):
    regex = re.compile("""%B(?P<cardnum>\d+)   # card number
                       \^(?P<lastname>[^/]+)   # last name
                       /(?P<firstname>[^\^]+)  # first name""",
                       re.VERBOSE)
    match = regex.match(line)
    return dict(cardnum=match.group("cardnum"), firstname=match.group("firstname").strip(),\
           lastname=match.group("lastname"))


class CardLogTable(QTableWidget):
    cardAdded = pyqtSignal('PyQt_PyObject', name='cardAdded')
    cardRemoved = pyqtSignal('PyQt_PyObject', name='cardremoved')
        
    def __init__(self, *args, **kw):
        super(QTableWidget, self).__init__(*args, **kw)
        
        self.verticalHeader().hide()
        self.verticalHeader().setResizeMode(QHeaderView.ResizeToContents)
        self.horizontalHeader().setResizeMode(QHeaderView.Stretch)
        self.horizontalHeader().setHighlightSections(False)
        self.setHorizontalHeaderLabels((("Card Number"), ("First Name"), ("Last Name")))
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setEditTriggers(QAbstractItemView.NoEditTriggers)


    def update_card_info(self, cardnum, firstname, lastname):
        # Should use a view or something rather than having the model
        # and view being coupled in the same class
        
        row_count = self.rowCount()

        # See if card num is already in model, if so delete
        for i in xrange(row_count):
            if self.item(i, 0).text() == str(cardnum):
                self.removeRow(i)
                self.cardRemoved.emit(dict(cardnum=str(cardnum), firstname=firstname,\
                                           lastname=lastname))
                return
        
        self.insertRow(row_count)
        self.setItem(row_count, 0, QTableWidgetItem(str(cardnum)))
        self.setItem(row_count, 1, QTableWidgetItem(str(firstname)))
        self.setItem(row_count, 2, QTableWidgetItem(str(lastname)))

        self.cardAdded.emit(dict(cardnum=str(cardnum), firstname=str(firstname),\
                                           lastname=str(lastname)))


    def remove_card(self, row):
        cardnum = self.item(row, 0).text()
        firstname = self.item(row, 1).text()
        lastname = self.item(row, 2).text()
        self.removeRow(row)

        self.cardRemoved.emit(dict(cardnum=cardnum, firstname=firstname,
                                   lastname=lastname))

    def remove_all(self):
        for row in xrange(self.rowCount() - 1, -1, -1):
            cardnum = self.item(row, 0).text()
            firstname = self.item(row, 1).text()
            lastname = self.item(row, 2).text()
            self.removeRow(row)

            self.cardRemoved.emit(dict(cardnum=cardnum, firstname=firstname,
                                       lastname=lastname))

class ScannerWindow(QMainWindow):    
    def __init__(self, *args, **kw):
        super(ScannerWindow, self).__init__(*args, **kw)
        self.resize(600, 400)

        widget = QWidget()
        
        self.log_table = CardLogTable(0, 3)
        self.log_table.cellDoubleClicked.connect(self.cell_double_clicked)

        self.card_scan_edit = QLineEdit()
        #self.card_scan_edit.setPlaceholderText("Click here and scan card")
        self.card_scan_edit.returnPressed.connect(self.card_entered)
        
        self.card_num_edit = QLineEdit()
        self.card_num_edit.setValidator(QRegExpValidator(QRegExp("\d*"),\
                                                         self.card_num_edit))
        self.card_num_edit.returnPressed.connect(self.manual_submit)
        #self.card_num_edit.setPlaceholderText("Card Number")
        
        self.first_name_edit = QLineEdit()
        self.first_name_edit.returnPressed.connect(self.manual_submit)
        #self.first_name_edit.setPlaceholderText("First Name")
        
        self.last_name_edit = QLineEdit()
        self.last_name_edit.returnPressed.connect(self.manual_submit)
        #self.last_name_edit.setPlaceholderText("Last Name")

        self.login_button = QPushButton("Login")
        self.login_button.clicked.connect(lambda checked: self.manual_submit())

        self.logout_all_button = QPushButton("Logout all users")
        self.logout_all_button.clicked.connect(lambda checked: self.logout_all())

        self.timer = QTimer()
        self.timer.timeout.connect(lambda: self.timeout())
        self.timer.start(1000)

        self.beep = QSound("beep.wav")
        
        layout = QGridLayout()
        layout.addWidget(self.card_num_edit, 0, 0)
        layout.addWidget(self.first_name_edit, 0, 1)
        layout.addWidget(self.last_name_edit, 0, 2)
        layout.addWidget(self.login_button, 0, 3)
        layout.addWidget(self.logout_all_button, 0, 4)
        layout.addWidget(self.card_scan_edit, 1, 0, 1, -1)
        layout.addWidget(self.log_table, 2, 0, 1, -1)

        widget.setLayout(layout)

        self.setCentralWidget(widget)
        self.card_scan_edit.setFocus()

    def timeout(self):
        try:
            with open("/dev/cardscanner", "r") as f:
                s = f.read()
                while len(s) > 0 and s[-1] != "\n":
                    s += f.read()
                if len(s) > 0:
                    logins = s.strip().split("\n")
                    for login in logins:
                        try:
                            card_info = parse_card_info(login)
                            self.log_table.update_card_info(card_info["cardnum"], card_info["firstname"],
                                                            card_info["lastname"])
                            self.beep.play()
                        except AttributeError as e:
                            print("Failed to read card info")
                        except Exception as e:
                            print("Unknown error while reading card")
                            traceback.print_exc()
        except:
            pass

    @pyqtSlot()
    def card_entered(self):
        try:
            card_info = parse_card_info(str(self.card_scan_edit.text()))
            self.log_table.update_card_info(card_info["cardnum"], card_info["firstname"],
                                            card_info["lastname"])
            self.beep.play()
        except AttributeError as e:
            print("Failed to read card info")
            #QMessageBox.warning(self, "Error", "Error reading card info")
        except Exception as e:
            print("Unknown error while reading card")
            traceback.print_exc()

        self.card_scan_edit.setText("")

    @pyqtSlot(int, int)
    def cell_double_clicked(self, row, col):
        self.log_table.remove_card(row)

    @pyqtSlot()
    def manual_submit(self):
        if self.card_num_edit.text() == "" or self.first_name_edit.text() == ""\
           or self.last_name_edit.text() == "":
            return
        
        cardnum = self.card_num_edit.text()
        self.card_num_edit.setText("")
        
        firstname = self.first_name_edit.text()
        self.first_name_edit.setText("")
        
        lastname = self.last_name_edit.text()
        self.last_name_edit.setText("")

        self.log_table.update_card_info(cardnum, firstname, lastname)
        
        self.card_num_edit.setFocus()

    @pyqtSlot()
    def logout_all(self):
        self.log_table.remove_all()
